/*
   MODULE NAME        ctgstats.h

   DESCRIPTIVE NAME   CICS Transaction Gateway Statistics
                      Application Programming Interface

                                           
   Licensed Materials - Property of IBM

   "Restricted Materials of IBM"

   5724-I81 5724-J09 5655-R25

   (c) Copyright IBM Corp. 2006, 2008 All Rights Reserved.

   US Government Users Restricted Rights - Use, duplication or
   disclosure restricted by GSA ADP Schedule Contract with
   IBM Corp.

   Status: Version 7 Release 2
                                         
*/

#ifndef _CTGSTATS_H
#define _CTGSTATS_H
/*********************************************************************/
/*                      Gateway Connection functions                 */
/*********************************************************************/
/**
 * CTG_GatewayToken_t represents a connection to a specific Gateway
 * daemon.
 */
typedef struct _CTG_GatewayToken_t * CTG_GatewayToken_t;

/**
 * CTG_ResultSetToken_t represents the results of an API call to
 * obtain Id or Stat data. It also maintains position in the result
 * position for getFirst/getNext use.
 */
typedef struct _CTG_ResultSetToken_t * CTG_ResultSetToken_t;

/**
 * Takes an integer for the target port number, a pointer to gateway
 * token and the address of a character pointer to be modified.
 * Establishes a connection to a local Gateway daemon statistics
 * protocol handler using the specified port number. If successful,
 * upon return the gateway token will represent the connection to the
 * specified Gateway daemon and will be required to interact with that
 * Gateway daemon on further API calls. protocolVersPtr will point to
 * a null-terminated string representing the protocol version (n_n_n_n)
 * of the statistics data supplied by the connected Gateway daemon. The
 * string storage is owned by the API, and the pointer remains valid
 * until the connection to the associated Gateway daemon has been
 * closed.
 *
 * @param port An int for the stats port of the target Gateway
 * daemon.
 * @param gwTokPtr Pointer to an int for the new Gateway Token.
 * @param protocolVersPtr The address of a character pointer to
 * be modified.
 * @return Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_BADPORT
 * CTG_STAT_ERR_NULLVERSPTR
 * CTG_STAT_ERR_BADRSOWNER
 * CTG_STAT_ERR_CONNECTFAILED
 * CTG_STAT_ERR_BADGWTOKLIST
 * CTG_STAT_ERR_NULLGWTOKPTR
 * CTG_STAT_ERR_NULLPARM
 * CTG_STAT_ERR_LOSTGWCON
 * CTG_STAT_ERR_COMMSFAIL
 */
int openGatewayConnection(int port,
                          CTG_GatewayToken_t * gwTokPtr,
                          char ** protocolVersPtr);

/**
 * Takes character pointer for the hostname, an integer for the target
 * port number, a pointer to gateway token and the address of a
 * character pointer to be modified.
 * Establishes a connection to a Gateway daemon statistics
 * protocol handler using the specified hostname and port number. If
 * successful, upon return the gateway token will represent the
 * connection to the specified Gateway daemon and will be required to
 * interact with that Gateway daemon on further API calls.
 * protocolVersPtr will point to a null-terminated string representing
 * the protocol version (n_n_n_n) of the statistics data supplied by the
 * connected Gateway daemon. The string storage is owned by the API, and
 * the pointer remains valid until the connection to the associated
 * Gateway daemon has been closed.
 *
 * @param address The character string containing the hostname or IP
 * address of the target Gateway daemon.
 * @param port An int for the stats port of the target Gateway
 * daemon.
 * @param gwTokPtr Pointer to an int for the new Gateway Token.
 * @param protocolVersPtr The address of a character pointer to
 * be modified.
 * @return Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_BADPORT
 * CTG_STAT_ERR_NULLVERSPTR
 * CTG_STAT_ERR_BADRSOWNER
 * CTG_STAT_ERR_CONNECTFAILED
 * CTG_STAT_ERR_BADGWTOKLIST
 * CTG_STAT_ERR_NULLGWTOKPTR
 * CTG_STAT_ERR_NULLPARM
 * CTG_STAT_ERR_LOSTGWCON
 * CTG_STAT_ERR_COMMSFAIL
 * CTG_STAT_ERR_BADHOST
 * CTG_STAT_ERR_NULLADDRESS
 */
int openRemoteGatewayConnection(char * address,
                                int port,
                                CTG_GatewayToken_t * gwTokPtr,
                                char ** protocolVersPtr);

/**
 * Attempts to free all resources held by the API, including open
 * Gateway daemon connections. This function maybe used by the API
 * program under normal shutdown circumstances, or in the event of a
 * severe error, enabling some form of controlled shutdown even when
 * references to gateway tokens have been lost.
 * Note: Copied result sets will not be freed, since the API does not
 * maintain a record of copied result sets.
 *
 * @param gwTokPtr The reference to the open Gateway connection
 * @return Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLGWTOK
 * CTG_STAT_ERR_BADGWTOK
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 * CTG_STAT_ERR_NULLGWTOKPTR
 * CTG_STAT_ERR_NULLRSTOKPTR
 * CTG_STAT_ERR_NULLRSTOK
 */
int closeGatewayConnection(CTG_GatewayToken_t * gwTokPtr);

/**
 * Attempts to free all resources held by the API, including open
 * Gateway connections. This function is provided to be called in
 * the event of a severe error, enabling some form of controlled
 * shutdown even if all gwTokens have been lost.
 *
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_BADGWTOKLIST
 */
int closeAllGatewayConnections();

/*********************************************************************/
/*                      Id-related functions                         */
/*********************************************************************/

/**
 * Takes a gateway token and a result set token pointer. Upon return,
 * the result set token will represent the set of resource group ids
 * currently available for the specified Gateway daemon.
 * The result set may be parsed with functions getFirstId and
 * getNextId, or used to generate a query string with getIdQuery().
 * Note: Depending upon when getResourceGroupIds() is called, dynamic
 * resource groups for specific CICS server may or may not be returned
 * in the list. The list of server resource group Ids may be obtained
 * directly via the appropriate resource group stat id.
 * Note: The caller may decide to take a local copy of the result set,
 * since it will re-cycled on the next API call for the same gwTok.
 *
 * @param gwTok The reference to the open Gateway connection.
 * @param resgrpIdRsTokPtr A pointer to a CTG_ResultSetToken_t
 * variable.
 * @return Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLGWTOK
 * CTG_STAT_ERR_NULLRSTOKPTR
 * CTG_STAT_ERR_NULLRSBUFFER
 * CTG_STAT_ERR_BADGWTOK
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 * CTG_STAT_ERR_LOSTGWCON
 * CTG_STAT_ERR_COMMSFAIL
 */
int getResourceGroupIds(CTG_GatewayToken_t gwTok,
                        CTG_ResultSetToken_t * resgrpIdRsTokPtr);

/**
 * Takes a gateway token and a result set token pointer. Upon return,
 * the result set token will represent the set of all available stat
 * ids on the specified gateway. The result set may be parsed with
 * functions getFirstId and getNextId, or used to generate a query
 * string with getIdQuery().
 * Note: The caller may decide to take a local copy of the result set,
 * since it will re-cycled on the next API call for the same gwTok.
 *
 * @param gwTok The reference to the open Gateway connection.
 * @param statIdRsTokPtr pointer to a CTG_ResultSetToken_t struct, to
 * be populated with details of the result.
 * @return Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLGWTOK
 * CTG_STAT_ERR_NULLRSTOKPTR
 * CTG_STAT_ERR_NULLRSBUFFER
 * CTG_STAT_ERR_BADGWTOK
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 * CTG_STAT_ERR_LOSTGWCON
 * CTG_STAT_ERR_COMMSFAIL
 */
int getStatIds(CTG_GatewayToken_t gwTok,
               CTG_ResultSetToken_t * statIdRsTokPtr);

/**
 * Takes a gateway token, a query string of stat group ids and a
 * result set token pointer. Upon return, the result set token will
 * represent the set of stat ids associated with the given query
 * string, for the specified Gateway daemon. The result set may be
 * parsed with functions getFirstId and getNextId, or used to
 * generate a query string with getIdQuery(). If the query
 * string is the empty string, then StatIds for all available
 * StatGroupIds will be returned.
 * Note: The caller may decide to take a local copy of the
 * result set, since it will re-cycled on the next API call for
 * the same gwTok.
 *
 * @param gwTok The reference to the open Gateway connection.
 * @param idQueryPtr pointer to a null-terminated string
 * containing StatGroupIds.
 * @param statIdRsTokPtr pointer to a CTG_ResultSetToken_t
 * struct, to be populated with details of the result.
 * @return Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLGWTOK
 * CTG_STAT_ERR_NULLRSTOKPTR
 * CTG_STAT_ERR_NULLQUERYSTRING
 * CTG_STAT_ERR_NULLRSBUFFER
 * CTG_STAT_ERR_BADGWTOK
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 * CTG_STAT_ERR_LOSTGWCON
 * CTG_STAT_ERR_COMMSFAIL
 */
int getStatIdsByStatGroupId(CTG_GatewayToken_t gwTok,
                            char * idQueryPtr,
                            CTG_ResultSetToken_t * statIdRsTokPtr);

/*********************************************************************/
/*                      Stats queries                                */
/*********************************************************************/

/**
 * Takes a gateway token, a query string of stat ids and a result set
 * token pointer. Upon return, the result set token will represent the
 * set of stat name/value pairs corresponding to the query string.
 * Note: The caller may decide to take a local copy of the result set,
 * since it will re-cycled on the next API call for the same gwTok.
 *
 * @param gwTok The reference to the open Gateway connection.
 * @param statRsTokPtr pointer to a CTG_ResultSetToken_t
 * variable to be populated with details of the result.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLGWTOK
 * CTG_STAT_ERR_RSTOKPTR
 * CTG_STAT_ERR_NULLRSBUFFER
 * CTG_STAT_ERR_BADGWTOK
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 * CTG_STAT_ERR_LOSTGWCON
 * CTG_STAT_ERR_COMMSFAIL
 */
int getStats(CTG_GatewayToken_t gwTok,
             CTG_ResultSetToken_t * statRsTokPtr);

/**
 * Takes a gateway token, a query string of stat ids and a
 * result set token pointer. Upon return, the result set token
 * will represent the set of stat name/value pairs corresponding
 * to the query string. If the query string is the empty string,
 * then Stats for all available StatIds will be returned.
 * Note: The caller may decide to take a local copy of the
 * result set, since it will re-cycled on the next API call for
 * the same gwTok.
 *
 * @param gwTok The reference to the open Gateway connection.
 * @param idQueryPtr pointer to a null-terminated string of comma
 * separated stat group ids.
 * @param statRsTokPtr pointer to a CTG_ResultSetToken_t struct, to be
 * populated with details of the result.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLGWTOK
 * CTG_STAT_ERR_NULLRSTOKPTR
 * CTG_STAT_ERR_NULLQUERYSTRING
 * CTG_STAT_ERR_NULLRSBUFFER
 * CTG_STAT_ERR_BADGWTOK
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 * CTG_STAT_ERR_LOSTGWCON
 * CTG_STAT_ERR_COMMSFAIL
 */
int getStatsByStatId(CTG_GatewayToken_t gwTok,
                     char * idQueryPtr,
                     CTG_ResultSetToken_t * statRsTokPtr);

/**
 * Takes a gateway token, a query string of stat group ids and a
 * result set token. Upon return, the result set token will represent
 * the set of stat name/value pairs corresponding to the query string.
 * If the query string is the empty string, then Stats for all
 * available StatGroupIds will be returned.
 * Note: The caller may decide to take a local copy of the
 * result set, since it will re-cycled on the next API call for
 * the same gwTok.
 *
 * @param gwTok The reference to the open Gateway connection.
 * @param idQueryPtr pointer to a null-terminated string of comma
 * separated stat group ids.
 * @param statRsTokPtr pointer to a CTG_ResultSetToken_t struct,
 * to be populated with details of the result
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLGWTOK
 * CTG_STAT_ERR_NULLRSTOKPTR
 * CTG_STAT_ERR_NULLQUERYSTRING
 * CTG_STAT_ERR_NULLRSBUFFER
 * CTG_STAT_ERR_BADGWTOK
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 * CTG_STAT_ERR_LOSTGWCON
 * CTG_STAT_ERR_COMMSFAIL
 */
int getStatsByStatGroupId(CTG_GatewayToken_t gwTok,
                          char * idQueryPtr,
                          CTG_ResultSetToken_t * statRsTokPtr);

/*********************************************************************/
/*                      ResultSet helper functions                   */
/*********************************************************************/
/**
 * CTG_IdData_t encapsulates a ResourceGroupId, ResourceID or StatId.
 * An id is represented by a length and a character pointer.
 */
typedef struct {
   char eyecatcher[8];
   char * queryElementPtr;
   int queryElementLen;
   int queryElementRC;
   int idLen;
   char * idPtr;
} CTG_IdData_t;

/**
 * CTG_StatData_t encapsulates a single statId and value pair. Each is
 * represented by a length and a character pointer.
 */
typedef struct {
   char eyecatcher[8];
   char * queryElementPtr;
   int queryElementLen;
   int queryElementRC;
   int statIdLen;
   char * statIdPtr;
   int statValueLen;
   char * statValuePtr;
} CTG_StatData_t;

/**
 * Takes a result set token, and the address of char pointer. The
 * pointer is set to point at a null-terminated string containing the
 * set of valid stat ids in the given result set. The set of ids are
 * in the format required for direct use as an query string. The string
 * storage is owned by the API, and the pointer remains valid until the
 * connection to the associated Gateway daemon has been closed (for API
 * owned result sets) or the result set has been freed (for user owned
 * result sets).
 *
 * @param anyRsTok is a CTG_ResultSetToken_t variable, populated
 * by one of the stat or id API functions.
 * @param idQueryPtr pointer to the IdType variable to populate.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_BADRSTOKEN
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 * CTG_STAT_ERR_NULLRSTOK
 * CTG_STAT_ERR_NULLPTR
 * CTG_STAT_ERR_NOQUERYIDS
 */
int getIdQuery(CTG_ResultSetToken_t anyRsTok,
               char ** idQueryPtr);

/**
 * Takes an id result set token and populates an IdDataType variable
 * with details of the first id in the set. If the result set is
 * empty or unpopulated, the idDataType variable will be unchanged.
 *
 * @param idRsTok is a CTG_ResultSetToken_t variable, populated
 * by one of the id API functions.
 * @param idDataPtr pointer to the CTG_IdData_t variable to populate.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLRSTOK
 * CTG_STAT_ERR_NULLIDDATAPTR
 * CTG_STAT_ERR_BADRSTYPE
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 * CTG_STAT_ERR_ENDOFRESULTSET
 */
int getFirstId(CTG_ResultSetToken_t idRsTok,
               CTG_IdData_t * idDataPtr);

/**
 * Takes an id result set and populates an CTG_IdData_t variable with
 * details of the next id in the set. If there are no further ids
 * in the result set, the CTG_IdData_t variable will be unchanged.
 *
 * @param idRsTok is a CTG_ResultSetToken_t variable, populated
 * by one of the id API functions.
 * @param idDataPtr pointer to the CTG_IdData_t variable to populate.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLRSTOK
 * CTG_STAT_ERR_NULLIDDATAPTR
 * CTG_STAT_ERR_BADRSTYPE
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 * CTG_STAT_ERR_ENDOFRESULTSET
 */
int getNextId(CTG_ResultSetToken_t idRsTok,
              CTG_IdData_t * idDataPtr);

/**
 * Takes a stat result set token and populates an StatDataType variable
 * with details of the first stat in the set. If the result set is
 * empty or unpopulated, the statDataType variable will be unchanged.
 *
 * @param statRsTok is a CTG_ResultSetToken_t variable,
 * populated by one of the stats API functions.
 * @param statDataPtr pointer to the CTG_StatData_t variable to
 * populate.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLRSTOK
 * CTG_STAT_ERR_NULLSTATDATAPTR
 * CTG_STAT_ERR_BADRSTYPE
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 * CTG_STAT_ERR_ENDOFRESULTSET
 */
int getFirstStat(CTG_ResultSetToken_t statRsTok,
                 CTG_StatData_t * statDataPtr);

/**
 * Takes a stat result set token and populates an StatDataType variable
 * with details of the next stat in the set. If there are no further
 * ids in the result set, the statDataType variable will be unchanged.
 *
 * @param statRsTok is a CTG_ResultSetToken_t variable,
 * populated by one of the stats API functions.
 * @param statDataPtr pointer to the CTG_StatData_t variable to
 * populate.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLRSTOK
 * CTG_STAT_ERR_NULLSTATDATAPTR
 * CTG_STAT_ERR_BADRSTYPE
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 * CTG_STAT_ERR_ENDOFRESULTSET
 */
int getNextStat(CTG_ResultSetToken_t statRsTok,
                CTG_StatData_t * statDataPtr);

/**
 * Takes a result set token as a source, and a result set token
 * pointer as a target. Duplicates a result set. Upon return, the
 * target result set token will represent an "application-owned" copy
 * of the source result set. This may be desirable if multiple API
 * calls are to be performed before processing of the results.
 * A application-owned result set may be parsed with the appropriate
 * result set functions in exactly the same way as an API-owner result
 * set.
 * Note: It is the responsibility of the caller to free all duplicated
 * result sets, using the freeResultSet function.
 *
 * @param srcRsTok is a CTG_ResultSetToken_t variable, populated
 * by one of the id or stats API functions.
 * @param tarRsTokPtr pointer to a CTG_ResultSetToken_t struct
 * to populate.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLRSTOK
 * CTG_STAT_ERR_NULLRSTOKPTR
 * CTG_STAT_ERR_NULLRSBUFFER
 * CTG_STAT_ERR_BADRSTYPE
 * CTG_STAT_ERR_PIDMISMATCH
 * CTG_STAT_ERR_TIDMISMATCH
 */
int copyResultSet(CTG_ResultSetToken_t srcRsTok,
                  CTG_ResultSetToken_t * tarRsTokPtr);

/**
 * Takes a result set token pointer for an application-owned result
 * set, created by copyResultSet(), and frees the storage associated
 * with that result set. Upon return, the result set token will be
 * set to null.
 * Note: Any attempt to free an API-owned result set will return an
 * error.
 *
 * @param rsTokPtr pointer to a user-owned result set.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLRSTOKPTR
 * CTG_STAT_ERR_APIRESULTSET
 */
int freeResultSet(CTG_ResultSetToken_t * rsTokPtr);


/*********************************************************************/
/*                      Utility functions                            */
/*********************************************************************/

/**
 * Takes the address of a character pointer to be modified. Upon
 * return, apiVersPtr will point to a null-terminated string
 * representing the version (n_n_n_n) of the active ctgstats DLL.
 * The string storage is owned by the API.
 *
 * @param apiVersPtr The address of a character pointer to
 * be modified. In the even of an error, apiVersPtr will be
 * unchanged.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLPTR
 */
int getStatsAPIVersion(char ** apiVersPtr);

/**
 * Takes a pointer to an int variable, and sets that variable
 * to the current API trace level.
 * Valid values are (defined in ctgstats.h), and are
 * CTG_STAT_TRACE_LEVEL0 = Disable all trace,
 * CTG_STAT_TRACE_LEVEL1 = Exception trace,
 * CTG_STAT_TRACE_LEVEL2 = Event trace,
 * CTG_STAT_TRACE_LEVEL3 = Entry/exit trace,
 * CTG_STAT_TRACE_LEVEL4 = Debug trace
 *
 * @param traceStatePtr pointer to the int to set with API trace
 * status.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_NULLPTR
 */
int getAPITraceLevel(int * traceStatePtr);

/**
 * Takes an int variable representing the desired trace level
 * and sets the API trace level accordingly. The default trace
 * destination will stderr, but this may be overridden by the API
 * function setTraceFile().
 * Valid values are (defined in ctgstats.h), and are
 * CTG_STAT_TRACE_LEVEL0 = Disable all trace,
 * CTG_STAT_TRACE_LEVEL1 = Exception trace,
 * CTG_STAT_TRACE_LEVEL2 = Event trace,
 * CTG_STAT_TRACE_LEVEL3 = Entry/exit trace,
 * CTG_STAT_TRACE_LEVEL4 = Debug trace
 *
 * @param traceState int containing required API trace status.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_TRACELEVEL
 */
int setAPITraceLevel(int traceState);

/**
 * Takes a character pointer to a null terminated string
 * representing the desired trace file destination. If the
 * specified file already exists, trace data will be appended.
 * If the API is unable to open the specified filename for
 * writing, then the trace destination will remain unchanged.
 * Specifying a null pointer will cause the trace destination to
 * revert to the default destination of stderr.
 *
 * @param traceFileNamePtr char pointer to a null terminated
 * string representing the desired trace path+filename.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_TRACELEVEL
 */
int setAPITraceFile(char * traceFileNamePtr);

/**
 * Takes a result set token and writes a representation of that result
 * set in printable characters to the current trace destination,
 * irrespective of trace settings.
 * For debug purposes only.
 *
 * @param rsTok Token for result set to dump.
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 * CTG_STAT_ERR_TRACEFILE
 * CTG_STAT_ERR_NULLRSTOK
 * CTG_STAT_ERR_NULLRSBUFFER
 * CTG_STAT_ERR_BADRSTYPE
 */
int dumpResultSet(CTG_ResultSetToken_t rsTok);

/**
 * Writes internal API debugging information to the trace destination,
 * irrespective of trace settings.
 * For debug purposes only.
 *
 * @returns Indication whether the call was successful or not.
 * Possible values include
 * CTG_STAT_OK
 * CTG_STAT_ERR_LOCKFAIL
 * CTG_STAT_ERR_MALLOCFAIL
 * CTG_STAT_ERR_ICONV
 */
int dumpState();

/*********************************************************************/
/*                      Return value definitions                     */
/*********************************************************************/

/* API function return codes */
/**
 * The API call completed successfully.
 */
#define CTG_STAT_OK                        0
/**
 * A specified Gateway Token was found to be NULL.
 */
#define CTG_STAT_ERR_NULLGWTOK          -100
/**
 * A Gateway Token was not recognised by the stats API. The
 * token is either invalid, or the connection to the gateway has
 * been lost and the token deleted.
 */
#define CTG_STAT_ERR_BADGWTOK           -101
/**
 * The connection to the Gateway has been lost. Possibly because
 * the Gateway daemon has been shut down.
 */
#define CTG_STAT_ERR_LOSTGWCON          -102
/**
 * One of the parameters passed to the method was null.
 */
#define CTG_STAT_ERR_NULLPARM           -103
/**
 * An invalid trace level was provided to setAPITraceLevel().
 */
#define CTG_STAT_ERR_TRACELEVEL         -104
/**
 * The pointer provided to store the protcol version from the
 * openGatewayConnection() call was NULL.
 */
#define CTG_STAT_ERR_NULLVERSPTR        -105
/**
 * An attempt was made to free a result set that belongs to the API
 * rather than the user.
 */
#define CTG_STAT_ERR_APIRESULTSET       -106
/**
 * The API was unable to communicate with the Gateway Daemon on the
 * port provided.
 */
#define CTG_STAT_ERR_BADPORT            -107
/**
 * The specified Gateway Token pointer was found to be NULL
 */
#define CTG_STAT_ERR_NULLGWTOKPTR       -108
/**
 * The specified pointer was found to be NULL
 */
#define CTG_STAT_ERR_NULLPTR            -109
/**
 * The specified result set buffer was found to be NULL.
 */
#define CTG_STAT_ERR_NULLRSBUFFER       -110
/**
 * The API was unable to allocate sufficient memory to complete the
 * operation.
 */
#define CTG_STAT_ERR_MALLOCFAIL         -111
/**
 * The result set does not contain any results.
 */
#define CTG_STAT_ERR_EMPTYRESULTSET     -112
/**
 * There are no more results in the result set.
 */
#define CTG_STAT_ERR_ENDOFRESULTSET     -113
/**
 * The Gateway Token list has become corrupt
 */
#define CTG_STAT_ERR_BADGWTOKLIST       -115
/**
 * An unknown result set owner type was used when trying to create a
 * result set.
 */
#define CTG_STAT_ERR_BADRSOWNER         -116
/**
 * The result set token was not recognised by the API.
 */
#define CTG_STAT_ERR_BADRSTOKEN         -117
/**
 * The openGatewayConnection call failed to establish a connection with
 * the Gateway daemon.
 */
#define CTG_STAT_ERR_CONNECTFAILED      -118
/**
 * The result set did not contain results that are statistics or ids.
 */
#define CTG_STAT_ERR_BADRSTYPE          -119
/**
 * The API was unable to create conversion descriptors for the native
 * code page.
 */
#define CTG_STAT_ERR_ICONV              -120
/**
 * The API was unable to create a lock during initialization.
 */
#define CTG_STAT_ERR_LOCKFAIL           -121
/**
 * An error occurred in the communications with the Gateway daemon. The
 * Gateway Token will be deleted.
 */
#define CTG_STAT_ERR_COMMSFAIL          -122
/**
 * The process that has attempted to use the Gateway token or API owned
 * result set is different to the one that created it.
 */
#define CTG_STAT_ERR_PIDMISMATCH        -123
/**
 * The thread that has attempted to use the Gateway token or API owned
 * result set is different to the one that created it.
 */
#define CTG_STAT_ERR_TIDMISMATCH        -124
/**
 * The specified result set token pointer was found to be NULL.
 */
#define CTG_STAT_ERR_NULLRSTOKPTR       -125
/**
 * The specified query string was found to be NULL.
 */
#define CTG_STAT_ERR_NULLQUERYSTRING    -126
/**
 * The specified result set token was found to be NULL.
 */
#define CTG_STAT_ERR_NULLRSTOK          -127
/**
 * The specified CTG_IdData_t pointer was found to be NULL.
 */
#define CTG_STAT_ERR_NULLIDDATAPTR      -128
/**
 * The specified CTG_StatData_t pointer was found to be NULL.
 */
#define CTG_STAT_ERR_NULLSTATDATAPTR    -129
/**
 * The API was unable to create a trace file at the specified
 * location.
 */
#define CTG_STAT_ERR_TRACEFILE          -130
/**
 * The result set does not contain any valid stat ids or
 * resource group ids.
 */
#define CTG_STAT_ERR_NOQUERYIDS         -131
/**
 * The hostname was not resolved to a valid IP address.
 */
#define CTG_STAT_ERR_BADHOST            -132
/**
 * The specified hostname was found to be NULL.
 */
#define CTG_STAT_ERR_NULLADDRESS        -133
/**
 * The connection failed to complete within 60 seconds.
 */
#define CTG_STAT_ERR_CONNECTTIMEOUT     -134
/**
 * The connection attempt was refused.
 */
#define CTG_STAT_ERR_CONNECTREFUSED     -135
/**
 * The connection attempt was unable to reach the specified Gateway
 * location.
 */
#define CTG_STAT_ERR_GATEWAYUNREACHABLE -136

/*********************************************************************/
/*                      CTG Stats constants                          */
/*********************************************************************/
/**
 * Trace level 0 disables all tracing.
 */
#define CTG_STAT_TRACE_LEVEL0              0
/**
 * Trace level 1 includes exception trace points only.
 */
#define CTG_STAT_TRACE_LEVEL1              1
/**
 * Trace level 2 includes event trace points + lower trace
 * levels.
 */
#define CTG_STAT_TRACE_LEVEL2              2
/**
 * Trace level 3 includes entry/exit trace points + lower trace
 * levels.
 */
#define CTG_STAT_TRACE_LEVEL3              3
/**
 * Trace level 4 includes debug trace points + lower trace
 * levels.
 */
#define CTG_STAT_TRACE_LEVEL4              4
/**
 *  Null Gateway daemon token
 */
#define CTG_STAT_NULL_GWTOK             NULL
/**
 * Null result set token
 */
#define CTG_STAT_NULL_RSTOK             NULL
/**
 * CICS TG Stats API version of compiled application
 */
#define CTG_STAT_API_VERSION       "1_1_0_1"

#endif /* ifndef _CTGSTATS_H */

/* Doxygen index page  */

/*! \mainpage CICS TG Statistics API Overview
 *
 * Documentation V7.2.0.2 
 *
 * This is the documentation for the C-language CICS Transaction
 * Gateway Statistics API, including all function prototypes with
 * a short and detailed descriptions, defined constants and possible
 * return codes.
 *
 * Information about the API and protocol version numbers can be found
 * in the product documentation for the the Statistics API, under
 * section 'API and Protocol version control'.
 *
 * \section classes Class Hierarchy
 * Note: This API is C-language based and not C++; therefore it does
 * not contain any C++ classes. The sections of documentation relating
 * to Class Hierarchy duplicate the information for Data Structures.
 */

