/*
   MODULE NAME        CTGCLIENT_ECI.H

   DESCRIPTIVE NAME   CICS Transaction Gateway ECI Version 2  
                      Application Programming Interface, ECI Parts

   V7.2.0.2  
                                       
   Licensed Materials - Property of IBM
   
   "Restricted Materials of IBM"
   
   5724-I81 5655-R25
   
   (c) Copyright IBM Corp. 2008 All Rights Reserved.
   
   US Government Users Restricted Rights - Use, duplication or
   disclosure restricted by GSA ADP Schedule Contract with
   IBM Corp.
   
   Status: Version 7 Release 2
                                     

   NOTES :-

   This header file is provided with the CICS Transaction Gateway. These
   products are available for a number of different operating environments
   and to ensure the correct sections of this header are included, one of
   the following constants must be defined prior to inclusion:

     CICS_W32  -  if building a Windows application
     CICS_AIX  -  if building an AIX application
     CICS_SOL  -  if building a Solaris application
     CICS_HPUX -  if building an HPUX application 
     CICS_LNX     if building a Linux application
     CICS_HPIT and CICS_HPUX -
                  if building an HPUX application on Itanium 
*/

#include "ctgclient.h"

#ifndef _CTG_ECI_H
#define _CTG_ECI_H

#if defined (CICS_W32) || defined(CICS_WNT)
  #define CICSCALL __cdecl
#else
  #define CICSCALL
#endif

#if defined(CICS_AIX)
#pragma options align=packed
#elif defined(CICS_HPUX)
#pragma pack 1
#else
#pragma pack(1)
#endif

/*****************************************************************************/
/** The Version 2 External Call Interface(ECI) parameter block.
 *  The CTG_ECI_PARMS structure is the parameter block used when 
 *  communicating with a CICS server. The parameter block fields are used 
 *  for input and output. To communicate with the CICS server through the 
 *  Gateway daemon use the CTG_ECI_Execute function. 
 *
 *  Set the ECI parameter block to nulls before setting the 
 *  input parameter fields.
 *
 *  Guidance on how to use the ECI to manage logical units of work
 *  can be found in the CICS Transaction Gateway Information Center: 
 *  Programming Guide, search for "Managing logical units of work".
 *
 *  The ECI parameter block is transmitted to the Gateway daemon without 
 *  any code page conversion. For more information on managing code pages 
 *  refer to the Information Center: Programming Guide, search for title 
 *  "Data conversion".
 *
 *  \note Text strings are NOT null-terminated.
 * 
 */
typedef struct
{
    /* Values used with eci_call_type, for detailed information refer to 
     * Field Docuementation eci_call_type */
   #define ECI_SYNC             1   
   /** \brief Identifies the type of ECI flow.
    *  A required input parameter, the call type identifies the
    *  type of the ECI flow. The valid values are : \li ECI_SYNC
    * 
    *   */
   signed short   eci_call_type;

   #define ECI_PROGRAM_NAME_LENGTH 8
   /** \brief  The CICS program to run.
    *  An input parameter, the CICS program name is required except
    *  when eci_extend_mode is ECI_BACKOUT.
    * 
    *  An 8 character field containing the name of the program to
    *  be called. Pad unused characters with spaces or nulls. To
    *  avoid code page conversion issues it is recommended to use
    *  uppercase characters A to Z, and the numeric characters 0 to
    *  9.  For more information on managing code pages refer to the
    *  Information Center: Programming Guide, search for title
    *  "Data conversion". */
   char  eci_program_name[ECI_PROGRAM_NAME_LENGTH]; 

   #define ECI_USERID_LENGTH       16
   /** \brief  A userid for Security checking. The eci_userid has a
    *  maximum length of 16 characters.  */
   char  eci_userid[ECI_USERID_LENGTH];        

   #define ECI_PASSWORD_LENGTH     16
   /** \brief  A password for Security checking. The eci_password
    *  has a maximum length of 16 characters. */
   char  eci_password[ECI_PASSWORD_LENGTH];   

   #define ECI_TRANSID_LENGTH      4
   /** \brief A CICS transaction identifier.
    * 
    *  A 4 character field optionally containing the ID of a CICS
    *  transaction. Pad unused characters with spaces or nulls. The
    *  parameter is ignored if eci_tpn is used (set to any value
    *  other than nulls). The use of this parameter depends on the
    *  client from which the request is sent. The value of
    *  eci_transid is converted from ASCII to EBCDIC, with no
    *  uppercase translation, and stored in EIBTRNID for the
    *  duration of the LINK to the program specified in the
    *  eci_program_name.
    *
    *  The called program runs under the mirror transaction CPMI,
    *  but is linked to under the eci_transid transaction name.
    *  This name is available to the called program for querying
    *  the transaction ID. Some servers use the transaction ID to
    *  determine security and performance attributes for the called
    *  program. In those servers, use this parameter to control the
    *  processing of your called programs. */
   char  eci_transid[ECI_TRANSID_LENGTH];  

   #define ECI_ABEND_CODE_LENGTH   4
   /** \brief CICS abend code.
   * 
   *   An output parameter, a 4 character field in which a CICS
   *   abend code is returned. If the transaction that executes the
   *   called program does not abend spaces are returned. */
   char  eci_abend_code[ECI_ABEND_CODE_LENGTH]; 

   /** \brief The data passed to and returned from the CICS program.
    * 
    *  An optional input parameter, a pointer to the data to be
    *  passed as the COMMAREA of the CICS program identified by
    *  eci_program_name. If no COMMAREA is required, supply a null
    *  pointer and set the eci_commarea_length to zero. The
    *  COMMAREA is also an output field, it can be used by the
    *  called program to return information to the application.
    *  A COMMAREA can only be returned from the program, if one was 
    *  sent */
   void *      eci_commarea;     

   /** \brief The length of the eci_commarea.
    * 
    *  The length of the COMMAREA in bytes. This is used by the
    *  interface as the maximum length of the eci_commarea. The
    *  COMMAREA lengths for sending and receiving data should be
    *  defined by commarea_outbound_length and
    *  commarea_inbound_length.
    * 
    *  This value must not exceed 32500. (Some client/server
    *  combinations might allow larger COMMAREAs, but this is
    *  not guaranteed to work.) */
   signed short   eci_commarea_length; 

   /** \brief Timeout in seconds.
   * 
   *   The time in seconds to wait for a response from the CICS
   *   server. A value of zero means that no limit is set.
   * 
   *   If timeout occurs, the conversation is abended. */
   signed short   eci_timeout;         

   /* Values used with eci_extend_mode, for detailed information refer to 
    * Field Docuementation eci_extend_mode */
   #define ECI_NO_EXTEND        0   
   #define ECI_EXTENDED         1 
   #define ECI_COMMIT           2
   #define ECI_BACKOUT          4 
   /** \brief Extend mode defines the behaviour of the ECI flow.
   * 
   *  A required input parameter that defines how a logical unit
   *  of work is processed.
   *
   *  The values for this field, shown by their symbolic names,
   *  are as follows:
   *  <UL>
   *  <li>     ECI_NO_EXTEND
   *   <OL><li> If the input eci_luw_token field is zero it is
   *           the only call for a logical unit of work.
   *       <li> If the input eci_luw_token field is not zero it
   *            is the last call for the specified logical unit of
   *            work.
   *  </OL>
   *  In each case, changes to recoverable resources are committed
   *  by a CICS end-of-task sync point, and the logical unit of
   *  work ends.
   *
   *  If you set eci_extend_mode to ECI_NO_EXTEND and
   *  eci_luw_token to 0, you will observe one request flowing
   *  from client to server and one reply flowing from server to
   *  client. The server sends the reply after the program
   *  specified in eci_program_name has been called and the changes
   *  made by that program have been committed.
   *
   *  <li>      ECI_EXTENDED
   *   <OL><li>  If the input eci_luw_token field is zero it
   *             is the first call for a logical unit of work that
   *             is to be continued.
   *       <li>  If the input eci_luw_token field is not zero ,
   *             this call is intended to continue the specified
   *             logical unit of work.
   *  </OL>
   *  In each case the logical unit of work continues after the
   *  called program completes successfully, and changes to
   *  recoverable resources remain uncommitted.
   *
   *   <li>      ECI_COMMIT
   *  \n Terminate the current logical unit of work, identified by
   *  the input eci_luw_token field, and commit all changes made to
   *  recoverable resources.
   *
   *   <li>      ECI_BACKOUT
   *  \n Terminate the logical unit of work identified by the input
   *  eci_luw_token field, and back out all changes made to
   *  recoverable resources.
   *  </UL>
   *
   *  For guidance on how to use the ECI to manage logical units of
   *  work refer to the CICS Transaction Gateway Information
   *  Center: Programming Guide, search for "Managing logical units
   *  of work".
   */
   signed short   eci_extend_mode;     

   /* Used with eci_luw_token to request a new logical unit of work */
   #define  ECI_LUW_NEW 0
   /**  \brief The token identifies the logical unit of work to
    *   which the ECI flow is associated.
    * 
    * An integer field is used for identifying the logical unit of
    * work to which a call belongs. The eci_luw_token must be set
    * to ECI_LUW_NEW or zero at the start of a logical unit of
    * work, regardless of whether the logical unit of work is going
    * to be extended. If the logical unit of work is to be
    * extended, the ECI updates eci_luw_token with a valid value on
    * the first call of the logical unit of work, the returned
    * value must be used as the input value for all subsequent
    * calls that are part of the same logical unit of work.
    * 
    * If the return code is not ECI_NO_ERROR, and the call was
    * continuing or ending an existing logical unit of work, this
    * field is used as output to report the condition of the
    * logical unit of work. If it is set to zero the logical unit
    * of work has ended and its updates have been backed out. If it
    * is nonzero, it is the same as the input value, the logical
    * unit of work is continuing, and its updates are still
    * pending. The LUW token must be set to zero when
    * eci_extend_mode is ECI_NO_EXTEND */
   signed int     eci_luw_token;

   /* ECI_VERSION_2 must be used */
   #define ECI_VERSION_2        3   
   /** \brief ECI version identifier.
    *  A required input parameter that identifies the version of
    *  the ECI for which the application is coded. The value
    *  ECI_VERSION_2 must be used to define this field. */
   signed short   eci_version;         

   #define ECI_SYSTEM_NAME_LENGTH  8 
   /** \brief Server name. An optional input parameter.
   *
   *   An 8 character field that specifies the name of the server
   *   to which the ECI request is to be directed. If supplied, it
   *   must be one of the server names returned by
   *   CTG_listSystems, with the exception that, if connected to a
   *   z/OS gateway, it is possible to pass the APPLID of a CICS
   *   region which the gateway will then communicate with using 
   *   EXCI.
   *   Pad unused characters with spaces or nulls.
   *   The value can be supplied whenever eci_luw_token is set
   *   to zero. (If it is supplied when eci_luw_token is not
   *   zero, it is ignored, because the server was established at
   *   the start of the logical unit of work.)
   *
   *   If the field is set to nulls, the default server is
   *   selected; the name of the chosen server is returned in
   *   this field. */
   char     eci_system_name[ECI_SYSTEM_NAME_LENGTH];        

   #define ECI_TPN_LENGTH          4 
   /** \brief A transaction identifier for the mirror transaction.
    * 
    *  Optional input parameter. A 4-character field that specifies
    *  the transaction ID that will be used in the server to
    *  process the ECI request. This transaction must be defined in
    *  the server as a CICS mirror transaction. If the field is not
    *  set, the default mirror transaction CPMI is used.
    * 
    *  If the ECI request is extended (see the description of
    *  eci_extend_mode), this parameter has a meaning only for the
    *  first request.
    * 
    *  If this field is used, the contents of eci_transid are
    *  ignored. */ 
   char     eci_tpn[ECI_TPN_LENGTH];  

   /** \brief Maximum data length to send to the Gateway. If
    *  commarea_outbound_length is set to 0 or is greater than
    *  eci_commarea_length, the value of eci_commarea_length is
    *  used instead of commarea_outbound_length. */
   signed short commarea_outbound_length;

   /** \brief Maximum data length to receive from the Gateway.
   *  If the commarea_inbound_length is set to 0 or is greater than
   *  eci_commarea_length, the value of eci_commarea_length is
   *  used instead of commarea_inbound_length. */
   signed short commarea_inbound_length;  

   /** \internal Reserved field */
   void * reserved1;      
} CTG_ECI_PARMS;


/*****************************************************************************/
/**
 *  A reference for an ECI Parameter block, CTG_ECI_Execute . 
 */
typedef CTG_ECI_PARMS *PCTG_ECI_PARMS;


/*****************************************************************************/
/**
 *  Sends an ECI request to the Gateway. 
 *  Equivalent to the flow() method of a com.ibm.ctg.client.JavaGateway object.
 *  
 * @param[in] gwTok The reference to the open Gateway connection
 * @param EciParms The reference to the CTG_ECI_PARMS structure, the 
 * fields of the structure are used for input and output. 
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li CTG_OK
 * \li CTG_ERR_LOCKFAIL  
 * \li CTG_ERR_MALLOCFAIL
 * \li CTG_ERR_BADGWTOKLIST
 * \li CTG_ERR_NULLGWTOKPTR
 * \li CTG_ERR_NULLPARM
 * \li CTG_ERR_LOSTGWCON
 * \li CTG_ERR_NULLECIPOINTER
 * \li CTG_ERR_UNKNOWN_REQUEST_TYPE
 * \li CTG_ERR_GATEWAY_CLOSED
 * \li CTG_ERR_WORK_WAS_REFUSED
 * \li CTG_ERR_GATEWAY_EXCEPTION
 * \li CTG_ERR_INVALID_REQUEST_TYPE
 *
 * Other possible values are all of the documented ECI return codes. 
 *   
*/
int CICSCALL CTG_ECI_Execute(CTG_ConnToken_t gwTok,
                             CTG_ECI_PARMS *EciParms);


/*********************************************************************/
/*                CICS TG ECI Return Code constants                  */
/*********************************************************************/
/**  ECI return code, no error. */
#define ECI_NO_ERROR                  0                                       

/**  ECI return code, bad data length. */
#define ECI_ERR_INVALID_DATA_LENGTH  -1
                                       
/**  ECI return code, the extend mode field is not valid.   */
#define ECI_ERR_INVALID_EXTEND_MODE  -2                                       

/**  ECI return code, no CICS system is available. */
#define ECI_ERR_NO_CICS              -3                                       

/**  ECI return code, the specified server is no
 *   longer available. */ 
#define ECI_ERR_CICS_DIED            -4                                       

/**  ECI return code, the value supplied for timeout is
 *   negative */ 
#define ECI_ERR_REQUEST_TIMEOUT      -5                                       

/**  ECI return code, the ECI call timed out. */
#define ECI_ERR_RESPONSE_TIMEOUT     -6                                       

/**  ECI return code, the CICS transaction abended. */
#define ECI_ERR_TRANSACTION_ABEND    -7                                       

/**  ECI return code, the value in Luw_Token is incorrect. */
#define ECI_ERR_LUW_TOKEN            -8                 

/**  ECI return code, an internal system error has occurred.*/
#define ECI_ERR_SYSTEM_ERROR         -9                                       

/**  ECI return code, the call type was not valid. */
#define ECI_ERR_INVALID_CALL_TYPE    -14                                      

/**  ECI return code, not enough resource to complete the
 *   request.  */
#define ECI_ERR_RESOURCE_SHORTAGE    -16                                      

/**  ECI return code, not enough communication resources
 *   to satisfy request. */
#define ECI_ERR_NO_SESSIONS          -17                                      

/**  ECI return code, the data area is not valid.  */
#define ECI_ERR_INVALID_DATA_AREA    -19                                      

/**  ECI return code, the version is not valid. */
#define ECI_ERR_INVALID_VERSION      -21                                      

/**  ECI return code, the requested server could not be located. */
#define ECI_ERR_UNKNOWN_SERVER       -22                                      

/**  ECI return code, an invalid combination of user ID and
 *   password was specified. */
#define ECI_ERR_SECURITY_ERROR       -27                                      

/**  ECI return code, reached limit on maximum number of
 *   systems. */
#define ECI_ERR_MAX_SYSTEMS          -28                                      

/**  ECI return code, not enough communication resources to
 *   satisfy request. */
#define ECI_ERR_MAX_SESSIONS         -29                                      

/**  ECI return code, an attempt was made to commit a logical
 *   unit of work, but the server was unable to commit the
 *   changes, and backed them out instead. */
#define ECI_ERR_ROLLEDBACK           -30                                      

#if defined(CICS_AIX)
#pragma options align=reset
#elif defined(CICS_HPUX) && defined(__cplusplus)
#pragma pack
#elif defined(CICS_HPUX) && defined(CICS_HPIT)
#pragma pack
#elif defined(CICS_HPUX)
#pragma HP_ALIGN HPUX_NATURAL
#else
#pragma pack()
#endif

#endif /* _CTG_ECI_H */

